class ReactionSystem {
    constructor(reactionId) {
        this.reactionId = reactionId;
        this.loadReactions();
    }

    async loadReactions() {
        try {
            const response = await fetch(`reaction/reactions.php?reaction_id=${this.reactionId}`);
            const data = await response.json();
            this.updateCounts(data.counts);
            
            // 個別のボタンの有効/無効を制御
            if (REACTION_CONFIG.limits.disableOnLimit) {
                this.updateButtonStates(data.limitedTypes);
            }
        } catch (error) {
            console.error('リアクション数の取得に失敗:', error);
        }
    }

    async addReaction(reactionType) {
        try {
            const response = await fetch('reaction/reactions.php', {
                method: 'POST',
                headers: { 'Content-Type': 'application/json' },
                body: JSON.stringify({
                    reaction_id: this.reactionId,
                    reaction_type: reactionType
                })
            });
            
            const result = await response.json();
            if (result.success) await this.loadReactions();
        } catch (error) {
            console.error('リアクションの追加に失敗:', error);
        }
    }

    updateCounts(counts) {
        Object.entries(counts).forEach(([type, count]) => {
            const buttonId = `reaction-${type}-${encodeURIComponent(this.reactionId)}`;
            const element = document.querySelector(`#${CSS.escape(buttonId)} .count`);
            if (element) {
                element.textContent = count;
            }
        });
    }

    updateButtonStates(limitedTypes) {
        const container = document.querySelector(`[reaction-id="${this.reactionId}"]`);
        if (!container) return;
    
        container.querySelectorAll('button').forEach(button => {
            if (button.classList.contains('expand-button')) return;
            
            // ボタンのIDをエンコードされた状態で比較するように修正
            const buttonParts = button.id.split('-');
            if (buttonParts.length < 3) return;
            
            const type = buttonParts[1];
            const encodedReactionId = encodeURIComponent(this.reactionId);
            const buttonEncodedReactionId = buttonParts.slice(2).join('-'); // IDに含まれる可能性のある追加のハイフンに対応
            
            if (buttonEncodedReactionId === encodedReactionId && limitedTypes.includes(type)) {
                button.disabled = true;
                button.title = 'このリアクションの制限に達しました';
                button.style.opacity = '0.5';
                button.style.cursor = 'not-allowed';
            } else if (buttonEncodedReactionId === encodedReactionId) {
                button.disabled = false;
                button.title = '';
                button.style.opacity = '1';
                button.style.cursor = 'pointer';
            }
        });
    }
}

class ReactionManager {
    constructor() {
        this.systems = new Map();
        this.initializeAllReactions();
    }

    initializeAllReactions() {
        document.querySelectorAll('[reaction-id]').forEach(element => {
            const reactionId = element.getAttribute('reaction-id');
            const buttonContainer = element.querySelector('.reaction-buttons');
            
            if (!buttonContainer) {
                console.warn(`Reaction buttons container not found for: ${reactionId}`);
                return;
            }
            
            const reactions = Object.entries(REACTION_CONFIG.reactions);
            const initialCount = REACTION_CONFIG.display?.initialButtons ?? 2;
            
            // 初期ボタンの作成
            reactions.slice(0, initialCount).forEach(([type, content]) => {
                this.createButton(type, content, reactionId, buttonContainer);
            });

            // 追加ボタンの処理
            const remainingButtons = reactions.slice(initialCount);
            if (remainingButtons.length > 0) {
                this.createExpandableButtons(reactionId, buttonContainer, remainingButtons);
            }

            this.systems.set(reactionId, new ReactionSystem(reactionId));
        });
    }

    createButton(type, content, reactionId, container) {
        const button = document.createElement('button');
        button.id = `reaction-${type}-${encodeURIComponent(reactionId)}`;
        
        const isImage = content.match(/\.(png|jpg|gif|svg)$/i);
        button.innerHTML = isImage 
            ? `<img src="${content}" class="reaction-icon" alt="${type}"> <span class="count">0</span>`
            : `${content} <span class="count">0</span>`;
        
        button.onclick = () => this.handleReaction(reactionId, type);
        container.appendChild(button);
    }

    createExpandableButtons(reactionId, container, hiddenButtons) {
        const expandButton = document.createElement('button');
        expandButton.className = 'expand-button';
        expandButton.innerHTML = '＋more';
        container.appendChild(expandButton);

        const extraContainer = document.createElement('div');
        extraContainer.className = 'extra-buttons';
        extraContainer.style.display = 'none';
        container.appendChild(extraContainer);

        hiddenButtons.forEach(([type, content]) => {
            this.createButton(type, content, reactionId, extraContainer);
        });

        expandButton.onclick = () => {
            extraContainer.style.display = extraContainer.style.display === 'none' ? 'flex' : 'none';
            expandButton.innerHTML = extraContainer.style.display === 'none' ? '＋more' : '－';
        };
    }

    handleReaction(reactionId, type) {
        const system = this.systems.get(reactionId);
        if (system) system.addReaction(type);
    }
}

document.addEventListener('DOMContentLoaded', () => {
    window.reactionManager = new ReactionManager();
});
