class ReactionSystem {
    constructor(reactionId) {
        this.reactionId = reactionId;
        this.limitReached = new Set(); // limitReachedを初期化
        this.loadReactions();
    }

    async loadReactions() {
        try {
            const response = await fetch(`reaction/reactions.php?reaction_id=${this.reactionId}`);
            const data = await response.json();
            this.updateCounts(data.counts);
            
            // limitReachedを更新
            this.limitReached = new Set(data.limitedTypes);
            
            // 個別のボタンの有効/無効を制御
            if (REACTION_CONFIG.limits.disableOnLimit) {
                this.updateButtonStates(data.limitedTypes);
            }
        } catch (error) {
            console.error('リアクション数の取得に失敗:', error);
        }
    }

    updateButtonStates(limitedTypes) {
        const container = document.querySelector(`[reaction-id="${this.reactionId}"]`);
        if (!container) return;

        container.querySelectorAll('button').forEach(button => {
            if (button.classList.contains('expand-button')) return;
            
            // ボタンのIDをエンコードされた状態で比較するように修正
            const buttonParts = button.id.split('-');
            if (buttonParts.length < 3) return;
            
            const type = buttonParts[1];
            const encodedReactionId = encodeURIComponent(this.reactionId);
            const buttonEncodedReactionId = buttonParts.slice(2).join('-'); // IDに含まれる可能性のある追加のハイフンに対応
            
            if (buttonEncodedReactionId === encodedReactionId && limitedTypes.includes(type)) {
                button.disabled = true;
                button.title = 'このリアクションの制限に達しました';
                button.style.opacity = '0.5';
                button.style.cursor = 'not-allowed';
            } else if (buttonEncodedReactionId === encodedReactionId) {
                button.disabled = false;
                button.title = '';
                button.style.opacity = '1';
                button.style.cursor = 'pointer';
            }
        });
    }


    async addReaction(reactionType) {
        if (this.limitReached.has(reactionType)) {
            return;
        }

        try {
            const response = await fetch('reaction/reactions.php', {
                method: 'POST',
                headers: {
                    'Content-Type': 'application/json',
                },
                body: JSON.stringify({
                    reaction_id: this.reactionId,
                    reaction_type: reactionType
                })
            });
            
            const result = await response.json();
            if (result.success) {
                await this.loadReactions();
            }
        } catch (error) {
            console.error('リアクションの追加に失敗:', error);
        }
    }

    updateCounts(counts) {
        Object.entries(REACTION_CONFIG.reactions).forEach(([type]) => {
            const countElement = document.getElementById(`count-${type}-${this.reactionId}`);
            if (!countElement) return;

            const count = counts[type] || 0;
            countElement.style.display = count > 0 ? 'flex' : 'none';
            
            const countSpan = countElement.querySelector('span');
            if (countSpan) countSpan.textContent = count;
        });
    }
}

class ReactionManager {
    constructor() {
        this.systems = new Map();
        this.initializeAllReactions();
    }

    initializeAllReactions() {
        document.querySelectorAll('[reaction-id]').forEach(element => {
            const reactionId = element.getAttribute('reaction-id');
            let buttonContainer = element.querySelector('.reaction-buttons');
            let countContainer = element.querySelector('.reaction-counts');
            
            if (!buttonContainer || !countContainer) {
                console.warn(`Reaction containers not found for reaction-id: ${reactionId}`);
                return;
            }
            
            const reactions = Object.entries(REACTION_CONFIG.reactions);
            const initialCount = REACTION_CONFIG.display?.initialButtons ?? 2; // デフォルト値として2を使用
            const initialButtons = reactions.slice(0, initialCount);
            const hiddenButtons = reactions.slice(initialCount);

            // 初期表示するボタンを生成
            initialButtons.forEach(([type, content]) => {
                this.createButton(type, content, reactionId, buttonContainer);
                this.createCountElement(type, content, reactionId, countContainer);
            });

            // 3つ以上ある場合は展開ボタンを追加
            if (hiddenButtons.length > 0) {
                const expandButton = document.createElement('button');
                expandButton.className = 'expand-button';
                expandButton.innerHTML = '＋more';
                expandButton.onclick = () => this.toggleExtraButtons(reactionId, expandButton);
                buttonContainer.appendChild(expandButton);

                // 残りのボタンを非表示で生成
                const extraContainer = document.createElement('div');
                extraContainer.className = 'extra-buttons';
                extraContainer.style.display = 'none';
                buttonContainer.appendChild(extraContainer);

                hiddenButtons.forEach(([type, content]) => {
                    this.createButton(type, content, reactionId, extraContainer);
                    this.createCountElement(type, content, reactionId, countContainer);
                });
            }

            const system = new ReactionSystem(reactionId);
            this.systems.set(reactionId, system);
        });
    }

    createContent(content, type, isCount = false) {
        const isImage = content.match(/\.(png|jpg|gif|svg)$/i);
        return isImage 
            ? `<img src="${content}" class="reaction-icon" alt="${type}">${isCount ? ' <span>0</span>' : ''}`
            : isCount ? `${content} <span>0</span>` : content;
    }

    createButton(type, content, reactionId, container) {
        const button = document.createElement('button');
        button.id = `reaction-${type}-${reactionId}`;
        button.innerHTML = this.createContent(content, type);
        button.onclick = () => this.handleReaction(reactionId, type);
        container.appendChild(button);
    }

    createCountElement(type, content, reactionId, container) {
        const countDiv = document.createElement('div');
        countDiv.id = `count-${type}-${reactionId}`;
        countDiv.className = 'count-item';
        countDiv.innerHTML = this.createContent(content, type, true);
        container.appendChild(countDiv);
    }

    toggleExtraButtons(reactionId, expandButton) {
        const buttonContainer = expandButton.parentElement;
        const extraContainer = buttonContainer.querySelector('.extra-buttons');
        
        if (extraContainer.style.display === 'none') {
            extraContainer.style.display = 'flex';
            expandButton.innerHTML = '－';
        } else {
            extraContainer.style.display = 'none';
            expandButton.innerHTML = '＋more';
        }
    }

    handleReaction(reactionId, type) {
        const system = this.systems.get(reactionId);
        if (system) {
            system.addReaction(type);
        }
    }
}

// ページ読み込み時に初期化
document.addEventListener('DOMContentLoaded', () => {
    window.reactionManager = new ReactionManager();
});
